@@ -1,13 +1,13 @@
 import argparse
-import os
+import fnmatch
 import shutil
 import sys
 import threading
-import traceback
 from copy import deepcopy
-from http.server import HTTPServer, SimpleHTTPRequestHandler
+from http.server import HTTPServer
+from pathlib import Path
 
-from kart.utils import KartMap, KartObserver
+from kart.utils import KartMap, KartObserver, KartRequesHandler
 
 
 class Kart:
@@ -27,13 +27,14 @@ class Kart:
         self.map_modifiers = map_modifiers
         self.renderers = renderers
         self.config = config
-        self.build_location = build_location
+        self.build_location = Path(build_location)
         self.lock = threading.Lock()
 
-    def prepare(self):
+    def prepare(self, start=True):
         self.site = {"config": self.config}
         for miner in self.miners:
-            miner.read_data()
+            if start:
+                miner.read_data()
             self.site.update(miner.collect())
         for modifier in self.content_modifiers:
             modifier.modify(self.site)
@@ -41,21 +42,19 @@ class Kart:
     def create_map(self):
         self.map = KartMap(self.config["base_url"])
         for mapper in self.mappers:
-            self.map.update(mapper.map(deepcopy(self.site)))
+            self.map.update(mapper.map(self.site))
         for modifier in self.map_modifiers:
             modifier.modify(self.map, self.site)
 
     def write(self):
         for renderer in self.renderers:
-            map = deepcopy(self.map)
-            site = deepcopy(self.site)
-            renderer.render(map, site, self.build_location)
+            renderer.render(self.map, self.site, self.build_location)
 
     def build(self):
         self.prepare()
         self.create_map()
         shutil.rmtree(self.build_location, ignore_errors=True)
-        os.makedirs(self.build_location, exist_ok=True)
+        self.build_location.mkdir(parents=True, exist_ok=True)
         self.write()
 
     # _site and _map are set and retrieved with a threading lock to prevent data races
@@ -64,63 +63,49 @@ class Kart:
     # preventing errors when serving the site during development
 
     def update_data(self):
-        self.site = {"config": self.config}
-        for miner in self.miners:
-            self.site.update(miner.collect())
-        for modifier in self.content_modifiers:
-            modifier.modify(self.site)
+        self.prepare(False)
         self.create_map()
         _site = deepcopy(self.site)
         _map = deepcopy(self.map)
         _urls = {}
+        _regexes = {}
         for slug, page in self.map.items():
             _urls[page["url"]] = slug
+            if "*" in page["url"] or "?" in page["url"]:
+                _regexes[page["url"]] = slug
         with self.lock:
             self._site = _site
             self._map = _map
             self._urls = _urls
+            self._regexes = _regexes
 
-    def get_url_data(self, url):
+    def serve_page(self, handler, url):
         with self.lock:
-            _map = self._map
-            _urls = self._urls
-        if url not in _urls.keys():
-            # TODO: generalize using regex
-            if url.split("/")[1] == "static":
-                url = "/static/*"
-            else:
-                url = "/*"
-        page = _map[_urls[url]]
-        return page
-
-    def get_global_data(self):
-        with self.lock:
-            return self._map, self._site
+            site_map = self._map
+            urls = self._urls
+            site = self._site
+            regexes = self._regexes
+        if url in urls:
+            page = site_map[urls[url]]
+        else:
+            pattern = next((x for x in regexes if fnmatch.fnmatch(url, x)))
+            page = site_map[regexes[pattern]]
+        if page:
+            self.renderer_dict[page["renderer"]].serve(handler, page, site_map, site)
 
     def serve(self, port=9000):
-        get_url_data = self.get_url_data
-        get_global_data = self.get_global_data
-        renderer_dict = {}
+        self.renderer_dict = {}
         observer = KartObserver()
         observer.action = self.update_data
         for miner in self.miners:
             miner.start_watching(observer)
         observer.start()
         for renderer in self.renderers:
-            renderer_dict[renderer.name] = renderer
+            self.renderer_dict[renderer.name] = renderer
             renderer.start_serving()
-
-        class RequestHandler(SimpleHTTPRequestHandler):
-            def do_GET(self):
-                page = get_url_data(self.path)
-                map, site = get_global_data()
-                if page:
-                    try:
-                        renderer_dict[page["renderer"]].serve(self, page, map, site)
-                    except Exception:
-                        print(traceback.format_exc())
-
-        httpd = HTTPServer(("", port), RequestHandler)
+        handler_class = KartRequesHandler
+        handler_class.action = self.serve_page
+        httpd = HTTPServer(("", port), handler_class)
         httpd.timeout = 0.1
         self.update_data()
         shutil.rmtree(self.build_location, ignore_errors=True)
