@@ -38,10 +38,14 @@ class DefaultCollectionMapper(Mapper):
         for object in collection.values():
             slug = f"{self.collection_name}.{object['slug']}"
             url = self.base_url + f"/{self.collection_name}/{object['slug']}/"
+            if "template" in object:
+                template = object["template"]
+            else:
+                template = self.template
             page = {
                 "url": url,
                 "data": object,
-                "template": self.template,
+                "template": template,
                 "renderer": "default_site_renderer",
             }
             urls[slug] = page
@@ -56,16 +60,20 @@ class DefaultPageMapper(Mapper):
     def map(self, site):
         for slug in site["pages"]:
             page = site["pages"][slug]
-            try:
+            if "url" in page:
                 url = page["url"]
-            except Exception:
+            elif slug == "index":
+                url = "/"
+            else:
                 url = f"/{slug}/"
-                if slug == "index":
-                    url = "/"
+            if "template" in page:
+                template = page["template"]
+            else:
+                template = self.template
             page = {
                 "url": url,
                 "data": page,
-                "template": self.template,
+                "template": template,
                 "renderer": "default_site_renderer",
             }
             self.urls[slug] = page
@@ -117,10 +125,10 @@ class DefaultTaxonomyMapper(Mapper):
             items = site[self.collection_name].values()
             filtered_items = []
             for item in items:
-                if type(item[self.taxonomy_name]) == str:
+                if isinstance(item[self.taxonomy_name], str):
                     if item[self.taxonomy_name] == taxonomy["slug"]:
                         filtered_items.append(item)
-                elif type(item[self.taxonomy_name]) == list:
+                elif isinstance(item[self.taxonomy_name], list):
                     if taxonomy["slug"] in item[self.taxonomy_name]:
                         filtered_items.append(item)
             try:
